---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--          Copyright (c) 2007, Raul Fechete, Georg Kienesberger         --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Unchecked_Deallocation;

with Ast2Cfg.Flow_Types; use Ast2Cfg.Flow_Types;
with Ast2Cfg.Cfgs; use Ast2Cfg.Cfgs;
with Ast2Cfg.Output; use Ast2Cfg.Output;

package body Ast2Cfg.Flow_World is

   -- Initialize
   procedure Init (World: in out World_Object) is
   begin
      Pkgs.Pkg_Class_Ptr_List.Init(World.Pkg_List);
   end Init;

   -- Free the world from memory. Time to hitch-hike!
   procedure Free (World: in out World_Object_Ptr) is

      procedure Destroy is new Ada.Unchecked_Deallocation(World_Object, World_Object_Ptr);
      procedure Recursion (Flow: in out Flow_Class_Ptr) is
         Succs: Flow_Class_Ptr_List.Object;
         Succ: Flow_Class_Ptr := null;
      begin
         Succs := Flow_Types.Get_Succs(Flow.all);
         Flow_Class_Ptr_List.Reset(Succs);
         while Flow_Class_Ptr_List.Has_Next(Succs) loop
            Flow_Class_Ptr_List.Get_Next(Succs, Succ);
            Recursion(Succ);
         end loop;

         if Flow.all in Cfgs.CFG_Object'Class then
            Cfgs.Free(Cfgs.CFG_Class_Ptr(Flow));
         elsif Flow.all in Pkgs.Pkg_Object'Class then
            Pkgs.Free(Pkgs.Pkg_Class_Ptr(Flow));
         else
            Put_Line(Error, "[Flow_World] Free: Object is not CFG or Pkg.");
         end if;
      end Recursion;

      Pkg: Pkgs.Pkg_Class_Ptr := null;
      Succs: Flow_Class_Ptr_List.Object;
      Succ: Flow_Class_Ptr := null;
   begin
      if World /= null then
         Pkgs.Pkg_Class_Ptr_List.Reset(World.Pkg_List);
         while Pkgs.Pkg_Class_Ptr_List.Has_Next(World.Pkg_List) loop
            Pkgs.Pkg_Class_Ptr_List.Get_Next(World.Pkg_List, Pkg);
            Succs := Flow_Types.Get_Succs(Flow_Object(Pkg.all));
            Flow_Class_Ptr_List.Reset(Succs);
            while Flow_Class_Ptr_List.Has_Next(Succs) loop
               Flow_Class_Ptr_List.Get_Next(Succs, Succ);
               Recursion(Succ);
            end loop;

            Pkgs.Free(Pkg);
         end loop;

         Pkgs.Pkg_Class_Ptr_List.Clear(World.Pkg_List);
         -- Destroy(World); -- NO! Currently the World_Object is not
                            -- dynamically allocated (see Transformation).
      end if;
   end Free;

   -- Add a new Package to the World
   procedure Add_Pkg (World: in out World_Object; Pkg: Pkgs.Pkg_Class_Ptr) is
   begin
      if (Pkg /= null) then
         Pkgs.Pkg_Class_Ptr_List.Append(World.Pkg_List, Pkg);
      end if;
   end Add_Pkg;

   -- Get all the Packages in the World
   function Get_Pkgs (World: in World_Object) return Pkgs.Pkg_Class_Ptr_List.Object is
   begin
      return World.Pkg_List;
   end Get_Pkgs;

end Ast2Cfg.Flow_World;
